#ifndef KEY_H
#define KEY_H

#include "bool.h"

/* Maximo tamao */
#define keyMaxLen 30

typedef struct skey * key;

key 
key_empty(void);
/*
DESC: Construye un dato vacio. 
PRE: { k = NULL }
   k = key_empty();
POS: {  Hay memoria ==> k --> empty 
      ! Hay memoria ==> k == NULL }
*/

key 
key_destroy(key k);
/*
DESC: Destructor del tipo. k cambia.
PRE: { k --> K }
   k = key_destroy(k);
POS: { Se destruye K /\ k == NULL }
*/

bool 
key_copyStr(key k,const char *s);
/*
DESC: Transforma el string en key y lo copia k. 
      Devuelve False si no hay memoria.
PRE: { k --> K /\ s --> S /\ len(S) <= keyMaxLen}
  b = key_copyStr(k,s);  
POS: {  Hay memoria ==>  b /\ k --> fromStr(S)   
      ! Hay memoria ==> !b /\ k --> K }   
*/

char *
key_toStr(const key k);
/*
DESC: Devuelve un nuevo string (despues de usarlo hay que liberarlo) que 
      representa la key.
PRE: { k --> K /\ s = NULL }
        s = key_toStr(k);
POS: {   Hay memoria ==> s --> toStr(K) 
       ! Hay memoria ==> s == NULL }
*/

key 
key_clone(const key k);
/*
DESC: Construye una copia.
PRE: { k1 --> K1 /\ k2 = NULL }
   k2 = key_clone(k1);  
POS: {  Hay memoria ==> k1 --> K1 /\ k2 --> K1 /\ k1 != k2
      ! Hay memoria ==> k2 == NULL /\ k1 --> K1 }
*/

bool
key_eq(const key k1, const key k2);
/*
DESC: Igualdad. Se puede usar en expresiones.
PRE: { k1 --> K1 /\ k2 --> K2 }
   b = key_eq(k1,k2);  
POS: { b == (K1 = K2) }
*/

bool
key_le(const key k1, const key k2);
/*
DESC: Menor. Se puede usar en expresiones.
PRE: { k1 --> K1 /\ k2 --> K2 }
   b = key_le(k1,k2);  
POS: { b == (K1 < K2) }
*/


#endif /* KEY_H */
